<?php

declare(strict_types=1);

/*
 * Copyright (C) 2023 Daniel Siepmann <coding@daniel-siepmann.de>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 * 02110-1301, USA.
 */

namespace DanielSiepmann\FediverseFeedWrappers;

class WrapperRegistry
{
    private array $wrappers;

    public function __construct(
        array $wrappers
    ) {
        $this->wrappers = $wrappers;
    }

    public function get(string $identifier): WrapperInterface
    {
        $wrapper = null;
        $className = $this->wrappers[$identifier] ?? '';
        if (class_exists($className)) {
            $wrapper = new $className();
        }

        if ($wrapper instanceof WrapperInterface) {
            return $wrapper;
        }

        throw new \InvalidArgumentException(
            sprintf(
                'No wrapper found for "%s", only available: %s.',
                $identifier,
                implode(', ', array_keys($this->wrappers))
            ),
            1676531371
        );
    }
}
