<?php

declare(strict_types=1);

/*
 * Copyright (C) 2023 Daniel Siepmann <coding@daniel-siepmann.de>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 * 02110-1301, USA.
 */

namespace DanielSiepmann\FediverseFeedWrappers\Tests\Unit;

use DanielSiepmann\FediverseFeedWrappers\WrapperRegistry;
use InvalidArgumentException;
use PHPUnit\Framework\TestCase;

/**
 * @covers \DanielSiepmann\FediverseFeedWrappers\WrapperRegistry
 */
class WrapperRegistryTest extends TestCase
{
    /**
     * @test
     */
    public function canBeCreated(): void
    {
        $subject = new WrapperRegistry([]);

        self::assertInstanceOf(
            WrapperRegistry::class,
            $subject
        );
    }

    /**
     * @test
     */
    public function throwsExceptionIfRequestedIdentifierIsNotConfigured(): void
    {
        $subject = new WrapperRegistry([]);

        $this->expectException(InvalidArgumentException::class);
        $this->expectExceptionCode(1676531371);
        $this->expectExceptionMessage('No wrapper found for "Identifier", only available: .');

        $subject->get('Identifier');
    }

    /**
     * @test
     */
    public function throwsExceptionIfRequestedIdentifierIsNotOfExpectedInterface(): void
    {
        $subject = new WrapperRegistry([
            'Identifier' => 'wrong type',
        ]);

        $this->expectException(InvalidArgumentException::class);
        $this->expectExceptionCode(1676531371);
        $this->expectExceptionMessage('No wrapper found for "Identifier", only available: Identifier.');

        $subject->get('Identifier');
    }
}
